/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.ws.params.simple;

import java.net.URL;
import java.util.Arrays;
import java.util.List;

import static java.util.Objects.requireNonNullElse;

public class BooleanOption extends Option
{
  public static class Builder extends Option.Builder
  {
    private boolean defaultValue = false;

    private boolean value = false;

    private String reprValue = null;

    public void setDefaultValue(Boolean defaultValue)
    {
      this.defaultValue = requireNonNullElse(defaultValue, false);
    }

    public void setValue(Boolean value)
    {
      this.value = requireNonNullElse(value, false);
    }

    public void setReprValue(String reprValue)
    {
      this.reprValue = reprValue;
    }

    @Override
    public void setPossibleValues(List<String> possibleValues)
    {
      throw new UnsupportedOperationException("cannot set possible values for boolean");
    }

    @Override
    public BooleanOption build()
    {
      return new BooleanOption(this);
    }
  }

  public static Builder newBuilder()
  {
    return new Builder();
  }

  protected BooleanOption(Builder builder)
  {
    super(builder);
    String reprValue = requireNonNullElse(builder.reprValue, name);
    defvalue = builder.defaultValue ? reprValue : null;
    value = builder.value ? reprValue : null;
    possibleVals = List.of(reprValue);
    displayVals = List.of(label);
  }

  public BooleanOption(String name, String descr, boolean required,
      Boolean defVal, Boolean val, URL link)
  {
    super(name, descr, required, (defVal != null && defVal ? name : null),
        (val != null && val ? name : null), Arrays.asList(name), link);
  }

  public BooleanOption(String name, String description, String label,
      boolean isrequired, Boolean defValue, String reprValue, URL link)
  {
    super(name, description, label, isrequired,
        defValue != null && defValue ? reprValue : null,
        defValue != null && defValue ? reprValue : null,
        Arrays.asList(reprValue), link);
  }

  public BooleanOption(String name, String description, String label,
      boolean isrequired, Boolean defValue, URL link)
  {
    this(name, description, label, isrequired, defValue, String.valueOf(true), link);
  }
}
